﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2018 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion
using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;

namespace HeuristicLab.Optimization {

  /// <summary>
  /// SpacingCalculator is defined as the standarddeviation of all d[i] 
  /// where d[i] is the minimum eukildean distance any point i has 
  /// to all OTHER points in the same front 
  /// </summary>
  public static class SpacingCalculator {

    public static double CalculateSpacing<TP>(IEnumerable<TP> qualities) where TP: IReadOnlyList<double> {
      if (qualities == null) throw new ArgumentNullException(nameof(qualities));
      var l = qualities.ToList();
      if (l.Count == 0) throw new ArgumentException("Front must not be empty.");
      if (l.Count == 1) return 0;

      var mat = l.ToMatrix();
      alglib.kdtreebuild(mat, mat.GetLength(0), mat.GetLength(1), 0, 2, out var tree);
      var summand = new double[2];
      var dists = new List<double>();
      foreach (var point in l) {
        alglib.kdtreequeryknn(tree, point.ToArray(), 2, true);
        alglib.kdtreequeryresultsdistances(tree, ref summand);
        dists.Add(summand[1]);
      }
      return dists.StandardDeviationPop();
    }
  }
}
